@@ -12,6 +12,7 @@ import xpart as xp
 
 from xobjects.hybrid_class import _build_xofields_dict
 
+from .general import _pkg_root
 from .internal_record import RecordIdentifier, RecordIndex, generate_get_record
 
 start_per_part_block = """
@@ -132,7 +133,11 @@ class MetaBeamElement(xo.MetaHybridClass):
         data['_xofields'] = xofields
 
         depends_on = []
-        extra_c_source = []
+        extra_c_source = [
+            _pkg_root.joinpath('headers','constants.h'),
+            _pkg_root.joinpath('headers','checks.h'),
+            _pkg_root.joinpath('headers','particle_states.h')
+        ]
         kernels = {}
 
         # Handle internal record
@@ -219,6 +224,10 @@ class MetaBeamElement(xo.MetaHybridClass):
 class BeamElement(xo.HybridClass, metaclass=MetaBeamElement):
 
     iscollective = None
+    isthick = False
+    behaves_like_drift = False
+    allow_backtrack = False
+    skip_in_loss_location_refinement = False
 
     def init_pipeline(self,pipeline_manager,name,partners_names=[]):
         self._pipeline_manager = pipeline_manager
