@@ -40,7 +40,7 @@ void LimitPolygon_track_local_particle(LimitPolygonData el,
 	// I assume that if I am in the function is because
 	// the particle is alive
     	if (!is_alive){
-           LocalParticle_set_state(part, 0);
+           LocalParticle_set_state(part, XT_LOST_ON_APERTURE);
 	}
 
     //end_per_particle_block
