@@ -3,20 +3,24 @@
 # Copyright (c) CERN, 2021.                 #
 # ######################################### #
 
+import io
 import math
 import logging
+import json
 from copy import deepcopy
 from pprint import pp
+from pathlib import Path
 
 import numpy as np
 
 import xobjects as xo
 import xpart as xp
+import xtrack as xt
 
 from .mad_loader import MadLoader
 from .beam_elements import element_classes
 from . import beam_elements
-from .beam_elements import Drift, BeamElement, Marker
+from .beam_elements import Drift, BeamElement, Marker, Multipole
 
 log = logging.getLogger(__name__)
 
@@ -34,15 +38,20 @@ def mk_class_namespace(extra_classes):
     return out
 
 
-_thick_element_types = (beam_elements.Drift, ) #TODO add DriftExact
-
 def _is_drift(element): # can be removed if length is zero
     return isinstance(element, (beam_elements.Drift,) )
 
-def _is_thick(element):
-    return  ((hasattr(element, "isthick") and element.isthick) or
-             (isinstance(element, _thick_element_types)))
+def _behaves_like_drift(element):
+    return hasattr(element, 'behaves_like_drift') and element.behaves_like_drift
 
+def _is_aperture(element):
+    return element.__class__.__name__.startswith('Limit')
+
+def _is_thick(element):
+    return  hasattr(element, "isthick") and element.isthick
+
+def _allow_backtrack(element):
+    return hasattr(element, 'allow_backtrack') and element.allow_backtrack
 
 def _next_name(prefix, names, name_format='{}{}'):
     """Return an available element name by appending a number"""
@@ -52,6 +61,39 @@ def _next_name(prefix, names, name_format='{}{}'):
         i += 1
     return name_format.format(prefix, i)
 
+def _dicts_equal(dict1, dict2):
+    if not isinstance(dict1, dict) or not isinstance(dict2, dict):
+        raise ValueError
+    if set(dict1.keys()) != set(dict2.keys()):
+        return False
+    for key in dict1.keys():
+        if hasattr(dict1[key], '__iter__'):
+            if not hasattr(dict2[key], '__iter__'):
+                return False
+            elif isinstance(dict1[key], dict):
+                if not isinstance(dict2[key], dict):
+                    return False
+                else:
+                    if not _dicts_equal(dict1[key], dict2[key]):
+                        return False
+            elif not np.array_equal(dict1[key], dict2[key]):
+                return False
+        elif dict1[key] != dict2[key]:
+            return False
+    return True
+
+def _apertures_equal(ap1, ap2):
+    if not _is_aperture(ap1) or not _is_aperture(ap2):
+        raise ValueError(f"Element {ap1} or {ap2} not an aperture!")
+    if ap1.__class__ != ap2.__class__:
+        return False
+    ap1 = ap1.to_dict()
+    ap2 = ap2.to_dict()
+    return _dicts_equal(ap1, ap2)
+
+def _lines_equal(line1, line2):
+    return _dicts_equal(line1.to_dict(), line2.to_dict())
+
 
 DEG2RAD = np.pi / 180.
 
@@ -156,6 +198,8 @@ class Line:
     corresponding beam element object.
     '''
 
+    _element_dict = None
+
     @classmethod
     def from_dict(cls, dct, _context=None, _buffer=None, classes=()):
         '''
@@ -198,6 +242,22 @@ class Line:
 
         return self
 
+    @classmethod
+    def from_json(cls, file, **kwargs):
+
+        if isinstance(file, io.IOBase):
+            dct = json.load(file)
+        else:
+            with open(file, 'r') as fid:
+                dct = json.load(fid)
+
+        if 'line' in dct.keys():
+            dct_line = dct['line']
+        else:
+            dct_line = dct
+
+        return cls.from_dict(dct_line, **kwargs)
+
     @classmethod
     def from_sequence(cls, nodes=None, length=None, elements=None, sequences=None, copy_elements=False,
                       naming_scheme='{}{}', auto_reorder=False, **kwargs):
@@ -309,6 +369,8 @@ class Line:
         skip_markers=False,
         merge_drifts=False,
         merge_multipoles=False,
+        expressions_for_element_types=None,
+        replace_in_expr=None
     ):
 
         """
@@ -330,7 +392,9 @@ class Line:
             skip_markers=skip_markers,
             merge_drifts=merge_drifts,
             merge_multipoles=merge_multipoles,
+            expressions_for_element_types=expressions_for_element_types,
             error_table=None,  # not implemented yet
+            replace_in_expr=replace_in_expr
             )
         line=loader.make_line()
         return line
@@ -340,7 +404,6 @@ class Line:
         from collections import defaultdict
         import xdeps as xd
 
-        # Extract globals values from madx
         _var_values = defaultdict(lambda: 0)
         _var_values.default_factory = None
 
@@ -365,6 +428,13 @@ class Line:
                                             dct['_var_management_data'][kk])
             manager.load(dct['_var_manager'])
 
+    @property
+    def config(self):
+        return self.tracker.config
+
+    @config.setter
+    def config(self, value):
+        self.tracker.config = value
 
     @property
     def vars(self):
@@ -376,6 +446,17 @@ class Line:
         if self._var_management is not None:
             return self._var_management['lref']
 
+    @property
+    def element_dict(self):
+        return self._element_dict
+
+    @element_dict.setter
+    def element_dict(self, value):
+        if self._element_dict is None:
+            self._element_dict = {}
+        self._element_dict.clear()
+        self._element_dict.update(value)
+
     def __init__(self, elements=(), element_names=None, particle_ref=None):
         if isinstance(elements, dict):
             element_dict = elements
@@ -435,11 +516,28 @@ class Line:
             else:
                 return [self.element_dict[nn] for nn in names]
 
-    def filter_elements(self, mask=None, exclude_types_starting_with=None):
+    def filter_elements(self, mask=None, exclude_types_starting_with=None,
+                        _make_tracker=True):
+        """
+        Return a new line with only the elements satisfying a given condition.
+        Other elements are replaced with Drifts.
 
+        Parameters
+        ----------
+        mask: list of bool
+            A list of booleans with the same length as the line.
+            If True, the element is kept, otherwise it is replaced with a Drift.
+        exclude_types_starting_with: str
+            If not None, all elements whose type starts with the given string
+            are replaced with Drifts.
         """
-        Replace with Drifts all elements satisfying a given condition.
-        """
+
+        if _make_tracker and self.tracker is not None:
+            new_tracker = self.tracker.filter_elements(mask=mask,
+                        exclude_types_starting_with=exclude_types_starting_with)
+            new_line = new_tracker.line
+            new_line.tracker = new_tracker
+            return new_line
 
         if mask is None:
             assert exclude_types_starting_with is not None
@@ -467,12 +565,29 @@ class Line:
 
         return new_line
 
-    def cycle(self, index_first_element=None, name_first_element=None):
+    def cycle(self, index_first_element=None, name_first_element=None,
+              _make_tracker=True):
 
         """
         Cycle the line to start from a given element.
+
+        Parameters
+        ----------
+        index_first_element: int
+            Index of the element to start from
+        name_first_element: str
+            Name of the element to start from
         """
 
+        if _make_tracker and self.tracker is not None:
+            new_tracker = self.tracker.cycle(
+                index_first_element=index_first_element,
+                name_first_element=name_first_element,
+            )
+            new_line = new_tracker.line
+            new_line.tracker = new_tracker
+            return new_line
+
         if ((index_first_element is not None and name_first_element is not None)
                or (index_first_element is None and name_first_element is None)):
              raise ValueError(
@@ -501,19 +616,49 @@ class Line:
         self.element_names = tuple(self.element_names)
 
     def unfreeze(self):
+        """
+        Unfreeze the line. This is useful if you want to modify the line
+        after it has been frozen (most likely by calling `build_tracker`).
+        """
         self.element_names = list(self.element_names)
         if hasattr(self, 'tracker') and self.tracker is not None:
             self.tracker._invalidate()
+            self.tracker = None
 
     def _frozen_check(self):
         if isinstance(self.element_names, tuple):
             raise ValueError(
                 'This action is not allowed as the line is frozen!')
 
+    def __getattr__(self, attr):
+        # If not in self look in self.tracker (if not None)
+        if self.tracker is not None and attr in dir(self.tracker):
+            return getattr(self.tracker, attr)
+        elif attr in dir(xt.Tracker):
+            # If in Tracker class, ask the used to build the tracker
+            raise AttributeError(
+                'The tracker is not built. Please call Line.build_tracker()')
+        else:
+            raise AttributeError(
+                f'Line object has no attribute `{attr}`')
+
+    def __dir__(self):
+        return list(set(object.__dir__(self) + dir(self.tracker)))
+
     def __len__(self):
         return len(self.element_names)
 
     def copy(self, _context=None, _buffer=None):
+        '''
+        Return a copy of the line.
+
+        Parameters
+        ----------
+        _context: xobjects.Context
+            xobjects context to be used for the copy
+        _buffer: xobjects.Buffer
+            xobjects buffer to be used for the copy
+        '''
 
         elements = {nn: ee.copy(_context=_context, _buffer=_buffer)
                                     for nn, ee in self.element_dict.items()}
@@ -530,24 +675,62 @@ class Line:
 
         return out
 
-
     def _var_management_to_dict(self):
         out = {}
         out['_var_management_data'] = deepcopy(self._var_management['data'])
         out['_var_manager'] = self._var_management['manager'].dump()
         return out
 
-    def to_dict(self):
+    def to_dict(self, include_var_management=True):
+
+        '''Return a dictionary representation of the line.
+
+        Parameters
+        ----------
+        include_var_management : bool, optional
+            If True (default) the dictionary will contain the information
+            needed to restore the line with deferred expressions.
+        '''
+
         out = {}
         out["elements"] = {k: el.to_dict() for k, el in self.element_dict.items()}
         out["element_names"] = self.element_names[:]
         if self.particle_ref is not None:
             out['particle_ref'] = self.particle_ref.to_dict()
-        if self._var_management is not None:
+        if self._var_management is not None and include_var_management:
+            if hasattr(self, '_in_multiline') and self._in_multiline:
+                raise ValueError('The line is part ot a MultiLine object. '
+                    'To save without expressions please use '
+                    '`line.to_dict(include_var_management=False)`.\n'
+                    'To save also the deferred expressions please save the '
+                    'entire multiline.\n ')
+
             out.update(self._var_management_to_dict())
         return out
 
+    def to_json(self, file, **kwargs):
+        '''Save the line to a json file.
+
+        Parameters
+        ----------
+        file: str or file-like object
+            The file to save to. If a string is provided, a file is opened and
+            closed. If a file-like object is provided, it is used directly.
+        **kwargs: dict
+            Additional keyword arguments are passed to the `Line.to_dict` method.
+        '''
+
+        if isinstance(file, io.IOBase):
+            json.dump(self.to_dict(**kwargs), file, cls=xo.JEncoder)
+        else:
+            with open(file, 'w') as fid:
+                json.dump(self.to_dict(**kwargs), fid, cls=xo.JEncoder)
+
     def to_pandas(self):
+        '''
+        Return a pandas DataFrame with the elements of the line.
+        '''
+
         elements = self.elements
         s_elements = np.array(self.get_s_elements())
         element_types = list(map(lambda e: e.__class__.__name__, elements))
@@ -567,6 +750,26 @@ class Line:
     def insert_element(self, index=None, element=None, name=None, at_s=None,
                        s_tol=1e-6):
 
+        '''Insert an element in the line.
+
+        Parameters
+        ----------
+        index: int, optional
+            Index of the element in the line. If `index` is provided, `at_s`
+            must be None.
+        element: xline.Element, optional
+            Element to be inserted. If `element` is provided, `name` must be
+            provided.
+        name: str
+            Name of the element. If `name` is provided, `element` must be
+            provided.
+        at_s: float, optional
+            Position of the element in the line. If `at_s` is provided, `index`
+            must be None.
+        s_tol: float, optional
+            Tolerance for the position of the element in the line.
+        '''
+
         if isinstance(index, str):
             assert index in self.element_names
             index = self.element_names.index(index)
@@ -622,7 +825,8 @@ class Line:
             for ii in range(i_first_drift_to_cut, i_last_drift_to_cut+1):
                 e_to_replace = self.element_dict[self.element_names[ii]]
                 if (not _is_drift(e_to_replace) and
-                    not e_to_replace.__class__.__name__.startswith('Limit')):
+                    not isinstance(e_to_replace, Marker) and
+                    not _is_aperture(e_to_replace)):
                     raise ValueError('Cannot replace active element '
                                         f'{self.element_names[ii]}')
 
@@ -672,6 +876,17 @@ class Line:
         return self
 
     def append_element(self, element, name):
+
+        '''Append element to the end of the lattice
+
+        Parameters
+        ----------
+        element : object
+            Element to append
+        name : str
+            Name of the element to append
+        '''
+
         self._frozen_check()
         assert name not in self.element_dict.keys()
         self.element_dict[name] = element
@@ -692,7 +907,20 @@ class Line:
 
     def get_s_position(self, at_elements=None, mode="upstream"):
 
-        '''Get s position for given elements'''
+        '''Get s position for given elements
+
+        Parameters
+        ----------
+        at_elements : str or list of str
+            Name of the element(s) to get s position for (default: all elements)
+        mode : str
+            "upstream" or "downstream" (default: "upstream")
+
+        Returns
+        -------
+        s : float or list of float
+            s position for given element(s)
+        '''
 
         assert mode in ["upstream", "downstream"]
         s_prev = 0
@@ -720,34 +948,64 @@ class Line:
             return s
 
     def remove_markers(self, inplace=True, keep=None):
-        if not inplace:
-            raise NotImplementedError
+        '''
+        Remove markers from the line
+
+        Parameters
+        ----------
+        inplace : bool
+            If True, remove markers from the line (default: True)
+        keep : str or list of str
+            Name of the markers to keep (default: None)
+        '''
+
+        if self._var_management is not None:
+            raise NotImplementedError('`remove_markers` not'
+                                      ' available when deferred expressions are'
+                                      ' used')
+
         self._frozen_check()
 
-        if isinstance(keep, str):
+        if keep is None:
+            keep = []
+        elif isinstance(keep, str):
             keep = [keep]
 
-        names = []
-        for ee, nn in zip(self.elements, self.element_names):
-            if isinstance(ee, Marker):
-                if keep is None or nn not in keep:
-                    continue
-            names.append(nn)
-
-        self.element_names = names
-        return self
-
-    def remove_inactive_multipoles(self, inplace=True):
-
-        self._frozen_check()
-
-        if not inplace:
-            raise NotImplementedError
-
         newline = Line(elements=[], element_names=[])
 
         for ee, nn in zip(self.elements, self.element_names):
-            if isinstance(ee, (beam_elements.Multipole)):
+            if isinstance(ee, Marker) and nn not in keep:
+                continue
+            newline.append_element(ee, nn)
+
+        if inplace:
+            self.element_names = newline.element_names
+            self.element_dict = newline.element_dict
+            return self
+        else:
+            return newline
+
+    def remove_inactive_multipoles(self, inplace=True, keep=None):
+        '''
+        Remove inactive multipoles from the line
+        '''
+
+        if self._var_management is not None:
+            raise NotImplementedError('`remove_inactive_multipoles` not'
+                                      ' available when deferred expressions are'
+                                      ' used')
+
+        self._frozen_check()
+
+        if keep is None:
+            keep = []
+        elif isinstance(keep, str):
+            keep = [keep]
+
+        newline = Line(elements=[], element_names=[])
+
+        for ee, nn in zip(self.elements, self.element_names):
+            if isinstance(ee, Multipole) and nn not in keep:
                 ctx2np = ee._context.nparray_from_context_array
                 aux = ([ee.hxl, ee.hyl]
                         + list(ctx2np(ee.knl)) + list(ctx2np(ee.ksl)))
@@ -755,79 +1013,182 @@ class Line:
                     continue
             newline.append_element(ee, nn)
 
+        if inplace:
+            self.element_names = newline.element_names
+            self.element_dict = newline.element_dict
+            return self
+        else:
+            return newline
 
-        self.element_names = newline.element_names
-        return self
+    def remove_zero_length_drifts(self, inplace=True, keep=None):
+        '''
+        Remove zero-length drifts from the line
+        '''
 
-    def remove_zero_length_drifts(self, inplace=True):
+        if self._var_management is not None:
+            raise NotImplementedError('`remove_zero_length_drifts` not'
+                                      ' available when deferred expressions are'
+                                      ' used')
 
         self._frozen_check()
 
-        if not inplace:
-            raise NotImplementedError
+        if keep is None:
+            keep = []
+        elif isinstance(keep, str):
+            keep = [keep]
 
         newline = Line(elements=[], element_names=[])
 
         for ee, nn in zip(self.elements, self.element_names):
-            if _is_drift(ee):
+            if _is_drift(ee) and nn not in keep:
                 if ee.length == 0.0:
                     continue
             newline.append_element(ee, nn)
 
-        self.element_names = newline.element_names
-        return self
+        if inplace:
+            self.element_names = newline.element_names
+            self.element_dict = newline.element_dict
+            return self
+        else:
+            return newline
 
-    def merge_consecutive_drifts(self, inplace=True):
+    def merge_consecutive_drifts(self, inplace=True, keep=None):
+        '''
+        Merge consecutive drifts into one drift
+        '''
+
+        if self._var_management is not None:
+            raise NotImplementedError('`merge_consecutive_drifts` not'
+                                      ' available when deferred expressions are'
+                                      ' used')
 
         self._frozen_check()
 
-        if not inplace:
-            raise NotImplementedError
+        if keep is None:
+            keep = []
+        elif isinstance(keep, str):
+            keep = [keep]
 
         newline = Line(elements=[], element_names=[])
 
         for ii, (ee, nn) in enumerate(zip(self.elements, self.element_names)):
             if ii == 0:
-                newline.append_element(ee, nn)
+                newline.append_element(ee.copy(), nn)
                 continue
 
-            if _is_drift(ee):
+            this_ee = ee if inplace else ee.copy()
+            if _is_drift(ee) and not nn in keep:
                 prev_nn = newline.element_names[-1]
                 prev_ee = newline.element_dict[prev_nn]
-                if _is_drift(prev_ee):
+                if _is_drift(prev_ee) and not prev_nn in keep:
                     prev_ee.length += ee.length
                 else:
-                    newline.append_element(ee, nn)
+                    newline.append_element(this_ee, nn)
             else:
-                newline.append_element(ee, nn)
+                newline.append_element(this_ee, nn)
 
-        self.element_dict.update(newline.element_dict)
-        self.element_names = newline.element_names
-        return self
+        if inplace:
+            self.element_names = newline.element_names
+            self.element_dict = newline.element_dict
+            return self
+        else:
+            return newline
 
-    def merge_consecutive_multipoles(self, inplace=True):
+    # For every occurence of three or more apertures that are the same,
+    # only separated by Drifts or Markers, this script removes the
+    # middle apertures
+    def remove_redundant_apertures(self, inplace=True, keep=None,
+                                  drifts_that_need_aperture=[]):
+        '''
+        Merge consecutive aperture checks by deleting the middle ones
+        '''
+
+        # TODO: this probably actually works, but better be safe than sorry
+        if self._var_management is not None:
+            raise NotImplementedError('`remove_redundant_apertures` not'
+                                      ' available when deferred expressions are'
+                                      ' used')
 
         self._frozen_check()
+
+        if keep is None:
+            keep = []
+        elif isinstance(keep, str):
+            keep = [keep]
+
+        aper_to_remove = []
+        # current aperture in loop
+        aper_0  = None
+        # previous aperture in loop (-1)
+        aper_m1 = None
+        # aperture before previous in loop (-2)
+        aper_m2 = None
+
+        for ee, nn in zip(self.elements, self.element_names):
+            if ee.__class__.__name__.startswith('Limit'):
+            # We encountered a new aperture, shift all previous
+                aper_m2 = aper_m1
+                aper_m1 = aper_0
+                aper_0  = nn
+            elif (not isinstance(ee, (Drift, Marker)) 
+            or nn in drifts_that_need_aperture):
+            # We are in an active element: all previous apertures
+            # should be kept in the line
+                aper_0  = None
+                aper_m1 = None
+                aper_m2 = None
+            if (aper_m2 is not None
+                and _apertures_equal(self.element_dict[aper_0], self.element_dict[aper_m1])
+                and _apertures_equal(self.element_dict[aper_m1], self.element_dict[aper_m2])
+                ):
+                # We found three consecutive apertures (with only Drifts and Markers
+                # in between) that are the same, hence the middle one can be removed
+                if aper_m1 not in keep:
+                    aper_to_remove = [*aper_to_remove, aper_m1]
+                    # Middle aperture removed, so the -2 shifts to the -1 position
+                    aper_m1 = aper_m2
+                    aper_m2 = None
+
+        if inplace:
+            newline = self
+        else:
+            newline = self.copy()
+
+        for name in aper_to_remove:
+            newline.element_names.remove(name)
+
+        return newline
+
+    def merge_consecutive_multipoles(self, inplace=True, keep=None):
+        '''
+        Merge consecutive multipoles into one multipole
+        '''
+
         if self._var_management is not None:
             raise NotImplementedError('`merge_consecutive_multipoles` not'
                                       ' available when deferred expressions are'
                                       ' used')
 
-        if not inplace:
-            raise NotImplementedError
+        self._frozen_check()
+
+        if keep is None:
+            keep = []
+        elif isinstance(keep, str):
+            keep = [keep]
 
         newline = Line(elements=[], element_names=[])
 
         for ee, nn in zip(self.elements, self.element_names):
-            if len(newline.elements) == 0:
+            if len(newline.element_names) == 0:
                 newline.append_element(ee, nn)
                 continue
 
-            if isinstance(ee, beam_elements.Multipole):
-                prev_ee = newline.elements[-1]
+            if isinstance(ee, Multipole) and nn not in keep:
                 prev_nn = newline.element_names[-1]
-                if (isinstance(prev_ee, beam_elements.Multipole)
+                prev_ee = newline.element_dict[prev_nn]
+                if (isinstance(prev_ee, Multipole)
                     and prev_ee.hxl==ee.hxl==0 and prev_ee.hyl==ee.hyl==0
+                    and prev_nn not in keep
                     ):
 
                     oo=max(len(prev_ee.knl), len(prev_ee.ksl),
@@ -842,7 +1203,7 @@ class Line:
                         ksl[ii]+=kk
                     for ii,kk in enumerate(ee._xobject.ksl):
                         ksl[ii]+=kk
-                    newee = beam_elements.Multipole(
+                    newee = Multipole(
                             knl=knl, ksl=ksl, hxl=prev_ee.hxl, hyl=prev_ee.hyl,
                             length=prev_ee.length,
                             radiation_flag=prev_ee.radiation_flag)
@@ -854,9 +1215,12 @@ class Line:
             else:
                 newline.append_element(ee, nn)
 
-        self.element_dict.update(newline.element_dict)
-        self.element_names = newline.element_names
-        return self
+        if inplace:
+            self.element_names = newline.element_names
+            self.element_dict = newline.element_dict
+            return self
+        else:
+            return newline
 
     def use_simple_quadrupoles(self):
         '''
@@ -908,20 +1272,38 @@ class Line:
 
         return elements, names
 
-    def check_aperture(self):
+    def check_aperture(self, needs_aperture=[]):
 
         '''Check that all active elements have an associated aperture.'''
 
         elements_df = self.to_pandas()
 
-        elements_df['is_aperture'] = elements_df.element_type.map(lambda s: s.startswith('Limit'))
+        elements_df['is_aperture'] = elements_df.name.map(
+                                            lambda nn: _is_aperture(self.element_dict[nn]))
         elements_df['i_aperture_upstream'] = np.nan
         elements_df['s_aperture_upstream'] = np.nan
         elements_df['i_aperture_downstream'] = np.nan
         elements_df['s_aperture_downstream'] = np.nan
-
         num_elements = len(self.element_names)
 
+        # Elements that don't need aperture
+        dont_need_aperture = {name: False for name in elements_df['name']}
+        for name in elements_df['name']:
+            ee = self.element_dict[name]
+            if _allow_backtrack(ee) and not name in needs_aperture:
+                dont_need_aperture[name] = True
+
+            # Correct isthick for elements that need aperture but have zero length.
+            # Use-case example: Before collimators are installed as EverestCollimator
+            # (or any BaseCollimator element), they are just Markers or Drifts. We
+            # want to enforce that they have an aperture when loading the line (when
+            # they are still Drifts), so their names are added to 'needs_aperture'.
+            # However, it is enough for them to have an upstream aperture as they are
+            # at this stage just Markers (and xcoll takes care of providing the down-
+            # stream aperture), so we mark them as thin.
+            if name in needs_aperture and hasattr(ee, 'length') and ee.length == 0:
+                elements_df.loc[elements_df['name']==name, 'isthick'] = False
+
         i_prev_aperture = elements_df[elements_df['is_aperture']].index[0]
         i_next_aperture = 0
 
@@ -932,13 +1314,7 @@ class Line:
                     f'Checking aperture: {round(iee/num_elements*100):2d}%  ',
                     end="\r", flush=True)
 
-            if elements_df.loc[iee, 'element_type'] == 'Drift':
-                continue
-
-            if elements_df.loc[iee, 'element_type'] == 'XYShift':
-                continue
-
-            if elements_df.loc[iee, 'element_type'] == 'SRotation':
+            if dont_need_aperture[elements_df.loc[iee, 'name']]:
                 continue
 
             if elements_df.loc[iee, 'is_aperture']:
@@ -1025,7 +1401,7 @@ def _deserialize_element(el, class_dict, _buffer):
 
 
 def _is_simple_quadrupole(el):
-    if not isinstance(el, beam_elements.Multipole):
+    if not isinstance(el, Multipole):
         return False
     return (el.radiation_flag == 0 and
             el.order == 1 and
@@ -1037,7 +1413,7 @@ def _is_simple_quadrupole(el):
 
 
 def _is_simple_dipole(el):
-    if not isinstance(el, beam_elements.Multipole):
+    if not isinstance(el, Multipole):
         return False
     return (el.radiation_flag == 0 and el.order == 0
             and not any(el.ksl) and not el.hyl)
