@@ -61,6 +61,36 @@ def compute_linear_normal_form(M, symplectify=False, only_4d_block=False,
                         responsiveness_tol=DEFAULT_MATRIX_RESPONSIVENESS_TOL,
                         stability_tol=DEFAULT_MATRIX_STABILITY_TOL):
 
+    '''
+    Compute the linear normal form of a 6x6 matrix M in the form:
+
+    M = W x Rot x W^-1
+
+    where Rot is a block diagonal matrix with 2x2 rotations.
+
+    Parameters
+    ----------
+    M : np.ndarray
+        6x6 matrix
+    symplectify : bool
+        If True, symplectify the matrix before computing the normal form
+    only_4d_block : bool
+        If True, only use the 4x4 block of M to compute the normal form
+    responsiveness_tol : float
+        Tolerance for the responsiveness of the matrix M
+    stability_tol : float
+        Tolerance for the stability of the matrix M
+
+    Returns
+    -------
+    W : np.ndarray
+        6x6 matrix
+    invW: np.ndarray
+        6x6 matrix
+    Rot : np.ndarray
+        6x6 matrix
+    '''
+
     if only_4d_block:
         M = M.copy()
         M[4:, :] = 0
