@@ -10,6 +10,7 @@
 import numpy as np
 
 from .general import Table
+from .twiss import TwissTable
 import xtrack as xt
 
 # Required functions
@@ -143,6 +144,8 @@ class SurveyTable(Table):
 
         return out
 
+SurveyTable.__getitem__ = TwissTable.__getitem__
+
 def _get_s_increments(elements):
     lengths = []
     for ee in elements:
