@@ -20,13 +20,13 @@ from .general import _pkg_root
 from .internal_record import (new_io_buffer,
                               start_internal_logging_for_elements_of_type,
                               stop_internal_logging_for_elements_of_type)
-from .line import Line
+from .line import Line, _is_thick
 from .pipeline import PipelineStatus
 from .survey import survey_from_tracker
 from .tracker_data import TrackerData
 from .twiss import (compute_one_turn_matrix_finite_differences,
                     find_closed_orbit, twiss_from_tracker)
-from .match import match_tracker
+from .match import match_tracker, closed_orbit_correction
 from .tapering import compensate_radiation_energy_loss
 from .prebuild_kernels import get_suitable_kernel, XT_PREBUILT_KERNELS_LOCATION
 
@@ -133,6 +133,22 @@ class Tracker:
         self.matrix_responsiveness_tol = lnf.DEFAULT_MATRIX_RESPONSIVENESS_TOL
         self.matrix_stability_tol = lnf.DEFAULT_MATRIX_STABILITY_TOL
 
+    @property
+    def matrix_responsiveness_tol(self):
+        return self.line.matrix_responsiveness_tol
+
+    @matrix_responsiveness_tol.setter
+    def matrix_responsiveness_tol(self, value):
+        self.line.matrix_responsiveness_tol = value
+
+    @property
+    def matrix_stability_tol(self):
+        return self.line.matrix_stability_tol
+
+    @matrix_stability_tol.setter
+    def matrix_stability_tol(self, value):
+        self.line.matrix_stability_tol = value
+
     def _init_track_with_collective(
         self,
         _context=None,
@@ -224,7 +240,7 @@ class Tracker:
                 pp.element_names = tempxtline.element_names
                 noncollective_xelements += pp.elements
             else:
-                if hasattr(pp, 'isthick') and pp.isthick:
+                if _is_thick(pp):
                     ldrift = pp.length
                 else:
                     ldrift = 0.
@@ -414,6 +430,9 @@ class Tracker:
         if verbose: print("Merge consecutive multipoles")
         line.merge_consecutive_multipoles()
 
+        if verbose: print("Remove redundant apertures")
+        line.remove_redundant_apertures()
+
         if verbose: print("Remove zero length drifts")
         line.remove_zero_length_drifts()
 
@@ -523,7 +542,8 @@ class Tracker:
         matrix_stability_tol=None,
         symplectify=False,
         reverse=False,
-        use_full_inverse=None
+        use_full_inverse=None,
+        strengths=False
         ):
 
         self._check_invalidated()
@@ -545,6 +565,13 @@ class Tracker:
         '''
         return match_tracker(self, vary, targets, **kwargs)
 
+    def correct_closed_orbit(self, reference, correction_config,
+                        solver=None, verbose=False, restore_if_fail=True):
+
+        closed_orbit_correction(self, reference, correction_config,
+                                solver=solver, verbose=verbose,
+                                restore_if_fail=restore_if_fail)
+
     def filter_elements(self, mask=None, exclude_types_starting_with=None):
 
         """
@@ -556,7 +583,8 @@ class Tracker:
         return self.__class__(
                  _buffer=self._buffer,
                  line=self.line.filter_elements(mask=mask,
-                     exclude_types_starting_with=exclude_types_starting_with),
+                     exclude_types_starting_with=exclude_types_starting_with,
+                     _make_tracker=False),
                  track_kernel=(self.track_kernel if not self.iscollective
                                     else self._supertracker.track_kernel),
                  element_classes=(self.element_classes if not self.iscollective
@@ -638,7 +666,8 @@ class Tracker:
         self._check_invalidated()
 
         cline = self.line.cycle(index_first_element=index_first_element,
-                                name_first_element=name_first_element)
+                                name_first_element=name_first_element,
+                                _make_tracker=False)
 
         if _buffer is None:
             if _context is None:
@@ -713,6 +742,11 @@ class Tracker:
                     local_particle_src=self.local_particle_src,
                 )
 
+    def track(self, *args, **kwargs):
+        pass
+        # This is a placeholder, it is replaced either by the collective
+        # tracker or the single particle tracker
+
     @property
     def particle_ref(self) -> xp.Particles:
         self._check_invalidated()
@@ -1684,6 +1718,17 @@ def freeze_longitudinal(tracker):
     finally:
         tracker.config = config
 
+@contextmanager
+def _temp_knobs(tracker, knobs: dict):
+    old_values = {kk: tracker.vars[kk]._value for kk in knobs.keys()}
+    try:
+        for kk, vv in knobs.items():
+            tracker.vars[kk] = vv
+        yield
+    finally:
+        for kk, vv in old_values.items():
+            tracker.vars[kk] = vv
+
 
 _freeze_longitudinal = freeze_longitudinal  # to avoid name clash with function argument
 
